@@ -34,7 +34,7 @@
 #include <errno.h>
 #include <limits.h>
 
-#define AIRSPY_RX_VERSION "1.0.3 3 Dec 2015"
+#define AIRSPY_RX_VERSION "1.0.5 23 April 2016"
 
 #ifndef bool
 typedef int bool;
@@ -91,6 +91,8 @@ int gettimeofday(struct timeval *tv, void* ignored)
 
 #define FLOAT32_EL_SIZE_BYTE (4)	/* 4bytes = 32bit float */
 #define INT16_EL_SIZE_BYTE (2)   /* 2bytes = 16bit int */
+#define INT12_EL_SIZE_BITS (12)
+#define INT8_EL_SIZE_BITS (8)
 
 #define FD_BUFFER_SIZE (16*1024)
 
@@ -118,6 +120,8 @@ int gettimeofday(struct timeval *tv, void* ignored)
 #define SENSITIVITY_GAIN_MAX (21)
 #define SAMPLES_TO_XFER_MAX_U64 (0x8000000000000000ull) /* Max value */
 
+#define MIN_SAMPLERATE_BY_VALUE (1000000)
+
 /* WAVE or RIFF WAVE file format containing data for AirSpy compatible with SDR# Wav IQ file */
 typedef struct 
 {
@@ -392,6 +396,18 @@ int rx_callback(airspy_transfer_t* transfer)
 				pt_rx_buffer = transfer->samples;
 				break;
 
+			case AIRSPY_SAMPLE_RAW:
+				if (packing_val)
+				{
+					bytes_to_write = transfer->sample_count * INT12_EL_SIZE_BITS / INT8_EL_SIZE_BITS;
+				}
+				else
+				{
+					bytes_to_write = transfer->sample_count * INT16_EL_SIZE_BYTE * 1;
+				}
+				pt_rx_buffer = transfer->samples;
+				break;
+
 			default:
 				bytes_to_write = 0;
 				pt_rx_buffer = NULL;
@@ -463,7 +479,7 @@ static void usage(void)
 		FREQ_HZ_MIN / FREQ_ONE_MHZ, FREQ_HZ_MAX / FREQ_ONE_MHZ, DEFAULT_FREQ_HZ / FREQ_ONE_MHZ);
 	printf("[-a sample_rate]: Set sample rate\n");
 	printf("[-t sample_type]: Set sample type, \n");
-	printf(" 0=FLOAT32_IQ, 1=FLOAT32_REAL, 2=INT16_IQ(default), 3=INT16_REAL, 4=U16_REAL\n");
+	printf(" 0=FLOAT32_IQ, 1=FLOAT32_REAL, 2=INT16_IQ(default), 3=INT16_REAL, 4=U16_REAL, 5=RAW\n");
 	printf("[-b biast]: Set Bias Tee, 1=enabled, 0=disabled(default)\n");
 	printf("[-v vga_gain]: Set VGA/IF gain, 0-%d (default %d)\n", VGA_GAIN_MAX, vga_gain);
 	printf("[-m mixer_gain]: Set Mixer gain, 0-%d (default %d)\n", MIXER_GAIN_MAX, mixer_gain);
@@ -617,6 +633,12 @@ int main(int argc, char** argv)
 						wav_nb_byte_per_sample = (wav_nb_bits_per_sample / 8);
 					break;
 
+					case 5:
+						sample_type_val = AIRSPY_SAMPLE_RAW;
+						wav_nb_bits_per_sample = 12;
+						wav_nb_channels = 1;
+						break;
+
 					default:
 						/* Invalid value will display error */
 						sample_type_val = SAMPLE_TYPE_MAX+1;
@@ -678,7 +700,7 @@ int main(int argc, char** argv)
 		sample_rate_val = sample_rate_u32;
 	}
 
-	bytes_to_xfer = samples_to_xfer * wav_nb_byte_per_sample * wav_nb_channels;
+	bytes_to_xfer = samples_to_xfer * wav_nb_bits_per_sample * wav_nb_channels / 8;
 
 	if (samples_to_xfer >= SAMPLES_TO_XFER_MAX_U64) {
 		printf("argument error: num_samples must be less than %s/%sMio\n",
@@ -705,6 +727,13 @@ int main(int argc, char** argv)
 	receiver_mode = RECEIVER_MODE_RX;
 	if( receive_wav ) 
 	{
+		if (sample_type_val == AIRSPY_SAMPLE_RAW)
+		{
+			printf("The RAW sampling mode is not compatible with Wave files\n");
+			usage();
+			return EXIT_FAILURE;
+		}
+
 		time (&rawtime);
 		timeinfo = localtime (&rawtime);
 		receiver_mode = RECEIVER_MODE_RX;
@@ -840,24 +869,47 @@ int main(int argc, char** argv)
 		}
 	}
 
-	airspy_get_samplerates(device, &count, 0);
-
-	if (sample_rate_val < 0 || sample_rate_val >= count)
-	{
-		printf("argument error: sample rate out of range\n");
+	result = airspy_set_sample_type(device, sample_type_val);
+	if (result != AIRSPY_SUCCESS) {
+		printf("airspy_set_sample_type() failed: %s (%d)\n", airspy_error_name(result), result);
 		airspy_close(device);
 		airspy_exit();
 		return EXIT_FAILURE;
 	}
 
+	airspy_get_samplerates(device, &count, 0);
+
 	supported_samplerates = (uint32_t *) malloc(count * sizeof(uint32_t));
 	airspy_get_samplerates(device, supported_samplerates, count);
-	wav_sample_per_sec = supported_samplerates[sample_rate_val];
+
+	if (sample_rate_val <= MIN_SAMPLERATE_BY_VALUE)
+	{
+		if (sample_rate_val < count)
+		{
+			wav_sample_per_sec = supported_samplerates[sample_rate_val];
+		}
+		else
+		{
+			free(supported_samplerates);
+			printf("argument error: unsupported sample rate\n");
+			airspy_close(device);
+			airspy_exit();
+			return EXIT_FAILURE;
+		}
+	}
+	else
+	{
+		wav_sample_per_sec = sample_rate_val;
+	}
+
 	free(supported_samplerates);
 
-	if (wav_nb_channels == 1)
-	{
-		wav_sample_per_sec *= 2;
+	result = airspy_set_samplerate(device, sample_rate_val);
+	if (result != AIRSPY_SUCCESS) {
+		printf("airspy_set_samplerate() failed: %s (%d)\n", airspy_error_name(result), result);
+		airspy_close(device);
+		airspy_exit();
+		return EXIT_FAILURE;
 	}
 
 	if (verbose)
@@ -888,22 +940,6 @@ int main(int argc, char** argv)
 		}
 	}
 
-	result = airspy_set_samplerate(device, sample_rate_val);
-	if( result != AIRSPY_SUCCESS ) {
-		printf("airspy_set_samplerate() failed: %s (%d)\n", airspy_error_name(result), result);
-		airspy_close(device);
-		airspy_exit();
-		return EXIT_FAILURE;
-	}
-
-	result = airspy_set_sample_type(device, sample_type_val);
-	if( result != AIRSPY_SUCCESS ) {
-		printf("airspy_set_sample_type() failed: %s (%d)\n", airspy_error_name(result), result);
-		airspy_close(device);
-		airspy_exit();
-		return EXIT_FAILURE;
-	}
-
 	result = airspy_set_rf_bias(device, biast_val);
 	if( result != AIRSPY_SUCCESS ) {
 		printf("airspy_set_rf_bias() failed: %s (%d)\n", airspy_error_name(result), result);
@@ -1006,7 +1042,7 @@ int main(int argc, char** argv)
 		(do_exit == false) )
 	{
 		float average_rate_now = average_rate * 1e-6f;
-		sprintf(str, "%2.2f", average_rate_now);
+		sprintf(str, "%2.3f", average_rate_now);
 		average_rate_now = 9.5f;
 		printf("Streaming at %5s MSPS\n", str);
 		if ((limit_num_samples == true) && (bytes_to_xfer == 0))
