@@ -295,18 +295,24 @@ static int prepare_transfers(airspy_device_t* device, const uint_fast8_t endpoin
 static void convert_samples_int16(uint16_t *src, int16_t *dest, int count)
 {
 	int i;
-	for (i = 0; i < count; i++)
+	for (i = 0; i < count; i += 4)
 	{
-		dest[i] = ((src[i] & 0xFFF) - 2048) << SAMPLE_SHIFT;
+		dest[i + 0] = (src[i + 0] - 2048) << SAMPLE_SHIFT;
+		dest[i + 1] = (src[i + 1] - 2048) << SAMPLE_SHIFT;
+		dest[i + 2] = (src[i + 2] - 2048) << SAMPLE_SHIFT;
+		dest[i + 3] = (src[i + 3] - 2048) << SAMPLE_SHIFT;
 	}
 }
 
 static void convert_samples_float(uint16_t *src, float *dest, int count)
 {
 	int i;
-	for (i = 0; i < count; i++)
+	for (i = 0; i < count; i += 4)
 	{
-		dest[i] = ((src[i] & 0xFFF) - 2048) * SAMPLE_SCALE;
+		dest[i + 0] = (src[i + 0] - 2048) * SAMPLE_SCALE;
+		dest[i + 1] = (src[i + 1] - 2048) * SAMPLE_SCALE;
+		dest[i + 2] = (src[i + 2] - 2048) * SAMPLE_SCALE;
+		dest[i + 3] = (src[i + 3] - 2048) * SAMPLE_SCALE;
 	}
 }
 
@@ -1292,6 +1298,28 @@ extern "C"
 		}
 	}
 
+	int ADDCALL airspy_spiflash_erase_sector(airspy_device_t* device, const uint16_t sector_num)
+	{
+		int result;
+		result = libusb_control_transfer(
+			device->usb_device,
+			LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_SPIFLASH_ERASE_SECTOR,
+			sector_num,
+			0,
+			NULL,
+			0,
+			0);
+
+		if (result != 0)
+		{
+			return AIRSPY_ERROR_LIBUSB;
+		}
+		else {
+			return AIRSPY_SUCCESS;
+		}
+	}
+
 	int ADDCALL airspy_spiflash_write(airspy_device_t* device, const uint32_t address, const uint16_t length, unsigned char* const data)
 	{
 		int result;
